@@ -8,14 +8,15 @@
 # 1. fix up GtkDrawingArea, GtkImage and GtkPixmap
 # 2. document some of this (maybe all of it)
 # 3. add wrappers for some of the GDK types.
-# 4. make more info about a GtkStyle available to Python code.
 
-# the noinit argument to the constructor of each object should not be used
-# when calling the constructor directly.  When it is set to something like
-# 1, the corresponding GTK object is not created.  This feature is used to
-# implement the new() function, and is used for methods that return Gtk
-# Objects, so that the corresponding Gtkinter objects have the correct
-# methods.
+# The _obj argument in each constructor is used to create a Gtkinter object
+# that coresponds to an already created GTK Object.  It should be possible
+# to call any of the Gtkinter object constructors something like this:
+# GtkContainer(_obj=gtk_obj) where gtk_obj is an already created GtkObject.
+#
+# To make the use of this system easier, you can use the _obj2inst function
+# to create a Gtkinter object of the correct type from a GtkObject.  This
+# is the function the new() command uses.
 
 from GTK import *
 import gtk; _gtk = gtk; del gtk
@@ -30,8 +31,8 @@ else:
 _gtk.gtk_rc_init()
 
 class GtkObject:
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 	def set(self, dict):
 		_filtprops(dict)
 		_gtk.gtk_object_set(self._o, dict)
@@ -45,27 +46,50 @@ class GtkObject:
 		if hasattr(v, '__class__') and v.__class__ in _supported:
 			v = v._o
 		_gtk.gtk_object_set(self._o, {key: v});
+	def flags(self, mask=None):
+		if mask:
+			return _gtk.GTK_OBJECT_FLAGS(self._o) & mask
+		else:
+			return _gtk.GTK_OBJECT_FLAGS(self._o)
+	def set_flags(self, flags):
+		_gtk.GTK_OBJECT_SET_FLAGS(self._o, flags)
+	def unset_flags(self, flags):
+		_gtk.GTK_OBJECT_UNSET_FLAGS(self._o, flags)
+
+	class __cnv:
+		def __init__(self, func):
+			self.func = func
+		def __call__(self, *args):
+			objtype = type(args[0])  #first arg is a GtkObject
+			a = list(args)
+			for i in range(len(args)):
+				if type(args[i]) == objtype:
+					a[i] = _obj2inst(args[i])
+			a = tuple(a)
+			return apply(self.func, a)
 	def connect(self, name, f):
-		_gtk.gtk_signal_connect(self._o, name, f)
+		callback = self.__cnv(f)
+		_gtk.gtk_signal_connect(self._o, name, callback.__call__)
 	def connect_after(self, name, f):
-		_gtk.gtk_signal_connect_after(self._o, name, f)
+		callback = self.__cnv(f)
+		_gtk.gtk_signal_connect_after(self._o, name, callback.__call__)
 	def emit_stop_by_name(self, sig):
 		_gtk.gtk_signal_emit_stop_by_name(self._o, sig)
 
 class GtkData(GtkObject):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 
 class GtkAdjustment(GtkData):
-	def __init__(self=0, lower=0, upper=0, step_incr=0, page_incr=0,
-			page_size=0, noinit=None):
-		if noinit: return
-		self._o = _gtk.gtk_adjustment_new(lower, upper, step_incr,
-			page_incr, page_size)
+	def __init__(self, value=0, lower=0, upper=0, step_incr=0, page_incr=0,
+			page_size=0, _obj=None):
+		if _obj: self._o = _obj; return
+		self._o = _gtk.gtk_adjustment_new(value, lower, upper,
+			step_incr, page_incr, page_size)
 
 class GtkWidget(GtkObject):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 	# extra parameter is so that these funcs can be used as signal funcs
 	def activate(self, obj=None):
 		_gtk.gtk_widget_activate(self._o)
@@ -151,8 +175,8 @@ class GtkWidget(GtkObject):
 		_gtk.gtk_widget_unrealize(self._o)
 	
 class GtkContainer(GtkWidget):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 	def add(self, child):
 		_gtk.gtk_container_add(self._o, child._o)
 	def block_resize(self, obj=None):
@@ -179,19 +203,19 @@ class GtkContainer(GtkWidget):
 		_gtk.gtk_container_unblock_resize(self._o)
 
 class GtkBin(GtkContainer):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 
 class GtkAlignment(GtkBin):
-	def __init__(self, xa=0, ya=0, xs=0, ys=0, noinit=None):
-		if noinit: return
+	def __init__(self, xa=0, ya=0, xs=0, ys=0, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_alignment_newe(xa, ya, xs, ys)
 	def set(self, xa, ya, xs, ys):
 		_gtk.gtk_alignment_set(self._o, xa, ya, xs, ys)
 
 class GtkButton(GtkBin):
-	def __init__(self, label=None, noinit=None):
-		if noinit: return
+	def __init__(self, label=None, _obj=None):
+		if _obj: self._o = _obj; return
 		if label == None:
 			self._o = _gtk.gtk_button_new()
 		else:
@@ -206,8 +230,8 @@ class GtkButton(GtkBin):
 		_gtk.gtk_button_leave(self._o)
 
 class GtkOptionMenu(GtkButton):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_option_menu_new()
 	def get_menu(self):
 		return _obj2inst(_gtk.gtk_option_menu_get_menu(self._o))
@@ -219,8 +243,8 @@ class GtkOptionMenu(GtkButton):
 		_gtk.gtk_option_menu_set_history(self._o, index)
 
 class GtkToggleButton(GtkButton):
-	def __init__(self, label=None, noinit=None):
-		if noinit: return
+	def __init__(self, label=None, _obj=None):
+		if _obj: self._o = _obj; return
 		if label == None:
 			self._o = _gtk.gtk_toggle_button_new()
 		else:
@@ -233,28 +257,32 @@ class GtkToggleButton(GtkButton):
 		_gtk.gtk_toggle_button_toggled(self._o)
 
 class GtkCheckButton(GtkToggleButton):
-	def __init__(self, label=None, noinit=None):
-		if noinit: return
+	def __init__(self, label=None, _obj=None):
+		if _obj: self._o = _obj; return
 		if label == None:
 			self._o = _gtk.gtk_check_button_new()
 		else:
 			self._o = _gtk.gtk_check_button_new_with_label(label)
 
 class GtkRadioButton(GtkCheckButton):
-	def __init__(self, group=None, label=None, noinit=None):
-		if noinit: return
+	def __init__(self, group=None, label=None, _obj=None):
+		if _obj: self._o = _obj; return
 		if label == None:
 			if group == None:
 				self._o = _gtk.gtk_radio_button_new()
 			else:
-				self._o = _gtk.gtk_radio_button_new(group)
+				self._o = _gtk.gtk_radio_button_new(group._o)
 		else:
-			self._o = _gtk.gtk_radio_button_new_with_label(group,
-				label)
+			if group == None:
+				self._o = _gtk.gtk_radio_button_new_with_label(
+					None, label)
+			else:
+				self._o = _gtk.gtk_radio_button_new_with_label(
+			    group._o, label)
 
 class GtkFrame(GtkBin):
-	def __init__(self, label="", noinit=None):
-		if noinit: return
+	def __init__(self, label="", _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_frame_new(label)
 	def set_label(self, label):
 		_gtk.gtk_frame_set_label(self._o, label)
@@ -264,25 +292,25 @@ class GtkFrame(GtkBin):
 		_gtk.gtk_frame_set_shadow_type(self._o, tp)
 
 class GtkAspectFrame(GtkFrame):
-	def __init__(self, xa=0, ya=0, ratio=1, obey_child=1, noinit=None):
-		if noinit: return
+	def __init__(self, xa=0, ya=0, ratio=1, obey_child=1, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_aspect_frame_new(xa, ya, ratio, obey_child)
 	def set(self, xa, ya, ratio, obey_child):
 		_gtk.gtk_aspect_frame_set(self._o, xa, ya, ratio, obey_child)
 
 class GtkEventBox(GtkBin):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_event_box_new()
 
 class GtkHandleBox(GtkBin):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_handle_box_new()
 
 class GtkItem(GtkBin):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 	def select(self, obj=None):
 		_gtk.gtk_item_select(self._o)
 	def deselect(self, obj=None):
@@ -291,8 +319,8 @@ class GtkItem(GtkBin):
 		_gtk.gtk_item_toggle(self._o)
 
 class GtkMenuItem(GtkItem):
-	def __init__(self, label=None, noinit=None):
-		if noinit: return
+	def __init__(self, label=None, _obj=None):
+		if _obj: self._o = _obj; return
 		if label == None:
 			self._o = _gtk.gtk_menu_item_new()
 		else:
@@ -311,12 +339,14 @@ class GtkMenuItem(GtkItem):
 		_gtk.gtk_menu_item_select(self._o)
 	def deselect(self, obj=None):
 		_gtk.gtk_menu_item_deselect(self._o)
-	def toggle(self, obj=None):
-		_gtk.gtk_menu_item_toggle(self._o)
+	def activate(self, obj=None):
+		_gtk.gtk_menu_item_activate(self._o)
+	def right_justify(self, obj=None):
+		_gtk.gtk_menu_item_right_justify(self._o)
 		
 class GtkCheckMenuItem(GtkMenuItem):
-	def __init__(self, label=None, noinit=None):
-		if noinit: return
+	def __init__(self, label=None, _obj=None):
+		if _obj: self._o = _obj; return
 		if label == None:
 			self._o = _gtk.gtk_check_menu_item_new()
 		else:
@@ -329,20 +359,24 @@ class GtkCheckMenuItem(GtkMenuItem):
 		_gtk.gtk_check_menu_item_toggled(self._o)
 
 class GtkRadioMenuItem(GtkCheckMenuItem):
-	def __init__(self, group=None, label=None, noinit=None):
-		if noinit: return
+	def __init__(self, group=None, label=None, _obj=None):
+		if _obj: self._o = _obj; return
 		if label == None:
 			if group == None:
 				self._o = _gtk.gtk_radio_menu_item_new()
 			else:
-				self._o = _gtk.gtk_radio_menu_item_new(group)
+				self._o =_gtk.gtk_radio_menu_item_new(group._o)
 		else:
-			self._o = _gtk.gtk_radio_menu_item_new_with_label(
-				group, label)
+			if group == None:
+				self._o=_gtk.gtk_radio_menu_item_new_with_label(
+					None, label)
+			else:
+				self._o=_gtk.gtk_radio_menu_item_new_with_label(
+					group._o, label)
 
 class GtkListItem(GtkItem):
-	def __init__(self, label=None, noinit=None):
-		if noinit: return
+	def __init__(self, label=None, _obj=None):
+		if _obj: self._o = _obj; return
 		if label == None:
 			self._o = _gtk.gtk_list_item_new()
 		else:
@@ -353,8 +387,8 @@ class GtkListItem(GtkItem):
 		_gtk.gtk_list_item_deselect(self._o)
 
 class GtkTreeItem(GtkItem):
-	def __init__(self, label=None, noinit=None):
-		if noinit: return
+	def __init__(self, label=None, _obj=None):
+		if _obj: self._o = _obj; return
 		if label == None:
 			self._o = _gtk.gtk_tree_item_new()
 		else:
@@ -373,8 +407,8 @@ class GtkTreeItem(GtkItem):
 		_gtk.gtk_tree_item_remove_subtree(self._o)
 
 class GtkViewport(GtkBin):
-	def __init__(self, ha=None, va=None, noinit=None):
-		if noinit: return
+	def __init__(self, ha=None, va=None, _obj=None):
+		if _obj: self._o = _obj; return
 		if ha or va:
 			self._o = _gtk.gtk_viewport_new(ha._o, va._o)
 		else:
@@ -391,8 +425,8 @@ class GtkViewport(GtkBin):
 		_gtk.gtk_viewport_set_shadow_type(self._o, tp)
 
 class GtkWindow(GtkBin):
-	def __init__(self, type=WINDOW_TOPLEVEL, title=None, noinit=None):
-		if noinit: return
+	def __init__(self, type=WINDOW_TOPLEVEL, title=None, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_window_new(type)
 		if title is not None:
 			self.set_title(title)
@@ -412,9 +446,9 @@ class GtkWindow(GtkBin):
 		_gtk.gtk_window_position(self._o, pos)
 
 class GtkColorSelectionDialog(GtkWindow):
-	def __init__(self, noinit=None):
-		if noinit: return
-		self._o = _gtk.gtk_color_selection_dialog_new()
+	def __init__(self, name="", _obj=None):
+		if _obj: self._o = _obj; return
+		self._o = _gtk.gtk_color_selection_dialog_new(name)
 	def __getattr__(self, attr):
 		attrs = {
 			'colorsel':_gtk.gtk_color_selection_dialog_get_colorsel,
@@ -429,13 +463,21 @@ class GtkColorSelectionDialog(GtkWindow):
 		raise AttributeError, attr
 
 class GtkDialog(GtkWindow):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_dialog_new()
+	def __getattr__(self, attr):
+		attrs = {
+			'vbox': _gtk.gtk_dialog_get_vbox,
+			'action_area': _gtk.gtk_dialog_get_action_area
+		}
+		if attrs.has_key(attr):
+			return _obj2inst(attrs[attr](self._o))
+		raise AttributeError, attr
 
 class GtkFileSelection(GtkWindow):
-	def __init__(self, title='', noinit=None):
-		if noinit: return
+	def __init__(self, title='', _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_file_selection_new(title)
 
 	def set_filename(self, filename):
@@ -457,12 +499,12 @@ class GtkFileSelection(GtkWindow):
 		return _obj2inst(attrs[attr](self._o))
 
 class GtkInputDialog(GtkWindow):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_input_dialog_new()
 
 class GtkBox(GtkContainer):
-	def __init__(self, noinit=None):
+	def __init__(self, _obj=None):
 		if nonint: return
 	def pack_start(self, child, expand=TRUE, fill=TRUE, padding=0):
 		_gtk.gtk_box_pack_start(self._o, child._o, expand,fill,padding)
@@ -476,13 +518,13 @@ class GtkBox(GtkContainer):
 		_gtk.gtk_box_set_spacing(self._o, spacing)
 
 class GtkHBox(GtkBox):
-	def __init__(self, homogeneous=0, spacing=0, noinit=None):
-		if noinit: return
+	def __init__(self, homogeneous=0, spacing=0, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_hbox_new(homogeneous, spacing)
 
 class GtkCombo(GtkHBox):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_combo_new()
 	def set_use_arrows(self, val):
 		_gtk.gtk_combo_set_use_arrows(self._o, val)
@@ -504,8 +546,8 @@ class GtkCombo(GtkHBox):
 		raise AttributeError, attr
 
 class GtkStatusbar(GtkHBox):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_statusbar_new()
 	def pop(self, sid):
 		_gtk.gtk_statusbar_pop(self._o, sid)
@@ -513,13 +555,13 @@ class GtkStatusbar(GtkHBox):
 		return _gtk.gtk_statusbar_push(self._o, text)
 
 class GtkVBox(GtkBox):
-	def __init__(self, homogeneous=0, spacing=0, noinit=None):
-		if noinit: return
+	def __init__(self, homogeneous=0, spacing=0, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_vbox_new(homogeneous, spacing)
 
 class GtkColorSelection(GtkVBox):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_color_selection_new()
 	def set_update_policy(self, p):
 		_gtk.gtk_color_selection_set_update_policy(self._o, p)
@@ -531,13 +573,13 @@ class GtkColorSelection(GtkVBox):
 		return _gtk.gtk_color_selection_get_color(self._o)
 
 class GtkGammaCurve(GtkVBox):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_gamma_curve_new()
 
 class GtkButtonBox(GtkBox):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 	def get_child_size_default(self):
 		return _gtk.gtk_button_box_get_child_size_default()
 	def get_child_ipadding_default(self):
@@ -564,8 +606,8 @@ class GtkButtonBox(GtkBox):
 		_gtk.gtk_button_box_set_child_ipadding(self._o, ix, iy)
 
 class GtkHButtonBox(GtkButtonBox):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_hbutton_box_new()
 	def get_spacing_default(self):
 		return _gtk.gtk_hbutton_box_get_spacing_default()
@@ -577,8 +619,8 @@ class GtkHButtonBox(GtkButtonBox):
 		_gtk.gtk_hbutton_box_set_layout_default(layout)
 
 class GtkVButtonBox(GtkButtonBox):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_vbutton_box_new()
 	def get_spacing_default(self):
 		return _gtk.gtk_vbutton_box_get_spacing_default()
@@ -590,8 +632,8 @@ class GtkVButtonBox(GtkButtonBox):
 		_gtk.gtk_vbutton_box_set_layout_default(layout)
 
 class GtkCList(GtkContainer):
-	def __init__(self, cols=1, titles=None, noinit=None):
-		if noinit: return
+	def __init__(self, cols=1, titles=None, _obj=None):
+		if _obj: self._o = _obj; return
 		if titles == None:
 			self._o = _gtk.gtk_clist_new(cols)
 		else:
@@ -640,6 +682,8 @@ class GtkCList(GtkContainer):
 		_gtk.gtk_clist_set_row_height(self._o, height)
 	def set_shift(self, row, col, v, h):
 		_gtk.gtk_clist_set_shift(self._o, row, col, v, h)
+	def set_selection_mode(self, mode):
+		_gtk.gtk_clist_set_selection_mode(self._o, mode)
 	def set_text(self, r, c, text):
 		_gtk.gtk_clist_set_text(self._o, r, c, text)
 	def thaw(self):
@@ -648,8 +692,8 @@ class GtkCList(GtkContainer):
 		_gtk.gtk_clist_unselect_row(self._o, row, col)
 
 class GtkFixed(GtkContainer):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_fixed_new()
 	def put(self, child, x, y):
 		_gtk.gtk_fixed_put(self._o, child._o, x, y)
@@ -657,9 +701,12 @@ class GtkFixed(GtkContainer):
 		_gtk.gtk_fixed_move(self._o, child._o, x, y)
 
 class GtkList(GtkContainer):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_list_new()
+	def get_selection(self):
+		l = _gtk.gtk_list_get_selection(self._o)
+		return map(_obj2inst, l)
 	def insert_items(self, items, pos):
 		_gtk.gtk_list_insert_items(self._o, items, pos)
 	def append_items(self, items):
@@ -687,8 +734,8 @@ class GtkList(GtkContainer):
 		_gtk.gtk_list_set_selection_mode(self._o, mode)
 
 class GtkMenuShell(GtkContainer):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 	def append(self, child):
 		_gtk.gtk_menu_shell_append(self._o, child._o)
 	def prepend(self, child):
@@ -699,8 +746,8 @@ class GtkMenuShell(GtkContainer):
 		_gtk.gtk_menu_shell_deactivate(self._o)
 
 class GtkMenuBar(GtkMenuShell):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_menu_bar_new()
 	def append(self, child):
 		_gtk.gtk_menu_bar_append(self._o, child._o)
@@ -710,8 +757,8 @@ class GtkMenuBar(GtkMenuShell):
 		_gtk.gtk_menu_bar_insert(self._o, child._o, pos)
 
 class GtkMenu(GtkMenuShell):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_menu_new()
 	def append(self, child):
 		_gtk.gtk_menu_append(self._o, child._o)
@@ -731,8 +778,8 @@ class GtkMenu(GtkMenuShell):
 		_gtk.gtk_menu_set_accelerator_table(self._o, table._at)
 
 class GtkNotebook(GtkContainer):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_notebook_new()
 	def append_page(self, child, tab):
 		_gtk.gtk_notebook_append_page(self._o, child._o, tab._o)
@@ -758,8 +805,8 @@ class GtkNotebook(GtkContainer):
 		_gtk.gtk_notebook_set_show_border(self._o, show)
 
 class GtkPaned(GtkContainer):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 	def add1(self, child):
 		_gtk.gtk_paned_add1(self._o, child._o)
 	def add2(self, child):
@@ -770,18 +817,18 @@ class GtkPaned(GtkContainer):
 		_gtk.gtk_paned_gutter_size(self._o, size)
 
 class GtkHPaned(GtkPaned):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_hpaned_new()
 
 class GtkVPaned(GtkPaned):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_vpaned_new()
 
 class GtkScrolledWindow(GtkContainer):
-	def __init__(self, hadj=None, vadj=None, noinit=None):
-		if noinit: return
+	def __init__(self, hadj=None, vadj=None, _obj=None):
+		if _obj: self._o = _obj; return
 		if not (vadj or hadj):
 			self._o = _gtk.gtk_scrolled_window_new()
 		else:
@@ -796,8 +843,8 @@ class GtkScrolledWindow(GtkContainer):
 		_gtk.gtk_scrolled_window_set_policy(self._o, hp, vp)
 
 class GtkTable(GtkContainer):
-	def __init__(self, rows=1, cols=1, homogeneous=0, noinit=None):
-		if noinit: return
+	def __init__(self, rows=1, cols=1, homogeneous=0, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_table_new(rows, cols, homogeneous)
 	def attach(self, child, la, ra, ta, ba, xoptions=EXPAND|FILL,
 			yoptions=EXPAND|FILL, xpadding=0, ypadding=0):
@@ -813,17 +860,24 @@ class GtkTable(GtkContainer):
 		_gtk.gtk_table_set_col_spacings(self._o, s)
 
 class GtkTree(GtkContainer):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_tree_new()
+	def get_selection(self):
+		return map(_obj2inst, _gtk.gtk_tree_get_selection(self._o))
 	def append(self, child):
 		_gtk.gtk_tree_append(self._o, child._o)
 	def prepend(self, child):
 		_gtk.gtk_tree_prepend(self._o, child._o)
 	def insert(self, child, pos):
 		_gtk.gtk_tree_insert(self._o, child._o, pos)
-#	def remove_item(self, child):
-#		_gtk.gtk_tree_remove_item(self._o, child._o)
+	def remove_item(self, child):
+		# fix this when the function starts existing
+		#_gtk.gtk_tree_remove_item(self._o, child._o)
+		self.remove_items([child])
+	def remove_items(self, children):
+		children = map(lambda x: x._o, children)
+		_gtk.gtk_tree_remove_items(self._o, children)
 	def clear_items(self, start, end):
 		_gtk.gtk_tree_clear_items(self._o, start, end)
 	def select_item(self, item):
@@ -845,8 +899,8 @@ class GtkTree(GtkContainer):
 
 class GtkToolbar(GtkContainer):
 	def __init__(self, orientation=ORIENTATION_HORIZONTAL,
-		     style=TOOLBAR_ICONS, noinit=None):
-		if noinit: return
+		     style=TOOLBAR_ICONS, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_toolbar_new(orientation, style)
         def append_item(self, text, tooltip, icon, callback):
 		_gtk.gtk_toolbar_append_item(self._o, text, tooltip, icon._o,
@@ -873,19 +927,21 @@ class GtkToolbar(GtkContainer):
 		_gtk.gtk_toolbar_set_orientation(self._o, orientation)
 	def set_style(self, style):
 		_gtk.gtk_toolbar_set_style(self._o, style)
+	def set_space_size(self, size):
+		_gtk.gtk_toolbar_set_space_size(self._o, size)
 	def set_tooltips(self, enable):
 		_gtk.gtk_toolbar_set_tooltips(self._o, enable)
 
 class GtkDrawingArea(GtkWidget):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_drawing_area_new()
 	def size(self, w, h):
 		_gtk.gtk_drawing_area_size(self._o, w, h)
 
 class GtkCurve(GtkDrawingArea):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_curve_new()
 	def reset(self):
 		_gtk.gtk_curve_reset(self._o)
@@ -897,8 +953,8 @@ class GtkCurve(GtkDrawingArea):
 		_gtk.gtk_curve_set_curve_type(self._o, tp)
 
 class GtkEntry(GtkWidget):
-	def __init__(self, maxlen=-1, noinit=None):
-		if noinit: return
+	def __init__(self, maxlen=-1, _obj=None):
+		if _obj: self._o = _obj; return
 		if maxlen == -1:
 			self._o = _gtk.gtk_entry_new()
 		else:
@@ -923,36 +979,36 @@ class GtkEntry(GtkWidget):
 		_gtk.gtk_entry_set_max_length(self._o, max)
 
 class GtkMisc(GtkWidget):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 	def set_alignment(self, xa, ya):
 		_gtk.gtk_misc_set_alignment(self._o, xa, ya)
 	def set_padding(self, xp, yp):
 		_gtk.gtk_misc_set_padding(self._o, xp, yp)
 
 class GtkArrow(GtkMisc):
-	def __init__(self, at=DIR_RIGHT, st=SHADOW_OUT, noinit=None):
-		if noinit: return
+	def __init__(self, at=DIR_RIGHT, st=SHADOW_OUT, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_arrow_new(at, st)
 	def set(self, at, st):
 		_gtk.gtk_arrow_set(self._o, at, st)
 
 class GtkImage(GtkMisc):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 	# XXX should have __init__, set, get
 
 class GtkPixmap(GtkMisc):
-	def __init__(self, parent=None, xpm=None, bg=None, noinit=None):
-		if noinit: return
+	def __init__(self, parent=None, xpm=None, bg=None, _obj=None):
+		if _obj: self._o = _obj; return
 		if bg:
 			self._o =_gtk.gtk_pixmap_new_from_xpm(parent._o,xpm,bg)
 		else:
 			self._o = _gtk.gtk_pixmap_new_from_xpm(parent._o, xpm)
 
 class GtkLabel(GtkMisc):
-	def __init__(self, label="", noinit=None):
-		if noinit: return
+	def __init__(self, label="", _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_label_new(label)
 	def set(self, str):
 		_gtk.gtk_label_set(self._o, str)
@@ -965,7 +1021,7 @@ class GtkPreview(GtkWidget):
 	def uninit(self):
 		_gtk.gtk_preview_uninit()
 	def __init__(self, tp=0):
-		if noinit: return
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_preview_new(tp)
 	def size(self, w, h):
 		_gtk.gtk_preview_size(self._o, w, h)
@@ -981,15 +1037,15 @@ class GtkPreview(GtkWidget):
 		_gtk.gtk_preview_set_reserved(reserved)
 
 class GtkProgressBar(GtkWidget):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_progress_bar_new()
 	def update(self, pcnt):
 		_gtk.gtk_progress_bar_update(self._o, pcnt)
 
 class GtkRange(GtkWidget):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 	def get_adjustment(self):
 		return _obj2inst(_gtk.gtk_range_get_adjustment(self._o))
 	def set_update_policy(self, pol):
@@ -1022,8 +1078,8 @@ class GtkRange(GtkWidget):
 		return _gtk.gtk_range_calc_value(self._o, pos)
 
 class GtkScale(GtkRange):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 	def set_digits(self, digits):
 		_gtk.gtk_scale_set_digits(self._o, digits)
 	def set_draw_value(self, draw_value):
@@ -1036,44 +1092,44 @@ class GtkScale(GtkRange):
 		_gtk.gtk_scale_draw_value(self._o)
 
 class GtkHScale(GtkScale):
-	def __init__(self, adj=None, noinit=None):
-		if noinit: return
+	def __init__(self, adj=None, _obj=None):
+		if _obj: self._o = _obj; return
 		if adj:
 			self._o = _gtk.gtk_hscale_new(adj._o)
 		else:
 			self._o = _gtk.gtk_hscale_new()
 
 class GtkVScale(GtkScale):
-	def __init__(self, adj=None, noinit=None):
-		if noinit: return
+	def __init__(self, adj=None, _obj=None):
+		if _obj: self._o = _obj; return
 		if adj:
 			self._o = _gtk.gtk_vscale_new(adj._o)
 		else:
 			self._o = _gtk.gtk_vscale_new()
 
 class GtkScrollbar(GtkRange):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 
 class GtkHScrollbar(GtkScrollbar):
-	def __init__(self, adj=None, noinit=None):
-		if noinit: return
+	def __init__(self, adj=None, _obj=None):
+		if _obj: self._o = _obj; return
 		if adj:
 			self._o = _gtk.gtk_hscrollbar_new(adj._o)
 		else:
 			self._o = _gtk.gtk_hscrollbar_new()
 
 class GtkVScrollbar(GtkScrollbar):
-	def __init__(self, adj=None, noinit=None):
-		if noinit: return
+	def __init__(self, adj=None, _obj=None):
+		if _obj: self._o = _obj; return
 		if adj:
 			self._o = _gtk.gtk_vscrollbar_new(adj._o)
 		else:
 			self._o = _gtk.gtk_vscrollbar_new()
 
 class GtkRuler(GtkWidget):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 	def set_metric(self, metric):
 		_gtk.gtk_ruler_set_metric(self._o, metric)
 	def set_range(self, lo, up, pos, max):
@@ -1084,32 +1140,32 @@ class GtkRuler(GtkWidget):
 		_gtk.gtk_ruler_draw_pos(self._o)
 
 class GtkHRuler(GtkRuler):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_hruler_new()
 
 class GtkVRuler(GtkRuler):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_vruler_new()
 
 class GtkSeparator(GtkWidget):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 
 class GtkHSeparator(GtkSeparator):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_hseparator_new()
 
 class GtkVSeparator(GtkSeparator):
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_vseparator_new()
 
 class GtkText(GtkWidget):
-	def __init__(self, hadj=None, vadj=None, noinit=None):
-		if noinit: return
+	def __init__(self, hadj=None, vadj=None, _obj=None):
+		if _obj: self._o = _obj; return
 		if not (hadj or vadj):
 			self._o = _gtk.gtk_text_new()
 		else:
@@ -1146,12 +1202,14 @@ _name2cls = {}
 _supported = map(lambda x: eval(x), filter(lambda x: x[:3] == 'Gtk', dir()))
 for m in _supported:
 	_name2cls[m.__name__] = m
-
+# strange widget type names
+_name2cls['color selection widget'] = GtkColorSelection
+_name2cls['color selection dialog'] = GtkColorSelectionDialog
 del m
 
 class GtkAcceleratorTable:
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._at = _gtk.gtk_accelerator_table_new()
 	def install(self, obj, signame, acc_key, acc_mods):
 		_gtk.gtk_accelerator_table_install(self._at, obj._o, signame,
@@ -1165,8 +1223,8 @@ class GtkAcceleratorTable:
 		_gtk.gtk_accelerator_table_set_mod_mask(self._at, mod_mask)
 
 class GtkTooltips:
-	def __init__(self, noinit=None):
-		if noinit: return
+	def __init__(self, _obj=None):
+		if _obj: self._o = _obj; return
 		self._tt = _gtk.gtk_tooltips_new()
 	def enable(self):
 		_gtk.gtk_tooltips_enable(self._tt)
@@ -1177,9 +1235,7 @@ class GtkTooltips:
 
 def _obj2inst(obj):
 	objname = _gtk.gtk_type_name(_gtk.GTK_OBJECT_TYPE(obj))
-	ret = _name2cls[objname](noinit=1)
-	ret._o = obj
-	return ret
+	return _name2cls[objname](_obj=obj)
 
 def _filtprops(props):
 	for k in props.keys():
